package main

import androidx.compose.foundation.background
import androidx.compose.foundation.border
import androidx.compose.foundation.layout.*
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.foundation.lazy.rememberLazyListState
import androidx.compose.material.Button
import androidx.compose.material.Icon
import androidx.compose.material.MaterialTheme
import androidx.compose.material.Text
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.ArrowBack
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.unit.dp

@Composable
fun glossaryDetailedPage(glossary: List<Word>, onBackClick: () -> Unit) {
    Column(
        modifier = Modifier.fillMaxSize(),
        verticalArrangement = Arrangement.Top, // Align content at the top
        horizontalAlignment = Alignment.CenterHorizontally
    ) {
        Text(text = "Détail du glossaire", style = MaterialTheme.typography.h3)

        Spacer(modifier = Modifier.height(16.dp))

        glossaryTable(glossary = glossary)

        Spacer(modifier = Modifier.height(16.dp))

        Button(
            onClick = { onBackClick() },
            colors = androidx.compose.material.ButtonDefaults.buttonColors(
                backgroundColor = customRedColor,
                contentColor = Color.White
            )
        ) {
            Icon(imageVector = Icons.Default.ArrowBack, contentDescription = null)
            Text("Retour")
        }
    }
}

// Faire un tableau qui affiche les mots et leur définition
@Composable
fun glossaryTable(glossary: List<Word>) {
    val listState = rememberLazyListState()
    Box(
        modifier = Modifier
            .height(500.dp)
            .padding(10.dp)
    ) {
    LazyColumn(state = listState, modifier = Modifier.padding(16.dp)) {
        item {
            headerRow()
        }
        items(glossary) { word ->
            glossaryDataRow(word)
            }
        }
    }
}


@Composable
fun headerRow() {
    val headers = listOf(
        "Mot",
        "Définition",
        "Contexte principal",
        "Contexte secondaire",
        "Mot lié",
        "Synonyme",
        "Antonyme"
    )

    Row(
        modifier = Modifier
            .fillMaxWidth()
            .background(customRedColor)
            .border(1.dp, Color.Black),
        verticalAlignment = Alignment.CenterVertically
    ) {
        headers.forEach { header ->
            Text(
                text = header,
                fontWeight = FontWeight.Bold,
                modifier = Modifier.weight(1f).padding(8.dp),
                color = Color.White
            )
        }
    }
}

@Composable
fun cellContent(text: String, modifier: Modifier = Modifier) {
    Text(
        text = text,
        modifier = modifier
    )
}

@Composable
fun glossaryDataRow(word: Word) {
    Row(
        modifier = Modifier
            .fillMaxWidth()
            .background(Color.White)
            .border(1.dp, Color.Black), // Ajouter une bordure à chaque ligne
        verticalAlignment = Alignment.CenterVertically
    ) {
        val modifier = Modifier.weight(1f).padding(8.dp)
        // Contenu de chaque cellule
        cellContent(word.name, modifier)
        cellContent(word.description, modifier)
        cellContent(word.mainContext, modifier)
        cellContent(word.secondaryContext, modifier)
        cellContent(word.relatedTo, modifier)
        cellContent(word.synonymous, modifier)
        cellContent(word.antonym, modifier)
    }
}