package main

import androidx.compose.desktop.ui.tooling.preview.Preview
import androidx.compose.foundation.VerticalScrollbar
import androidx.compose.foundation.border
import androidx.compose.foundation.layout.*
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.foundation.lazy.rememberLazyListState
import androidx.compose.foundation.rememberScrollbarAdapter
import androidx.compose.material.*
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.Delete
import androidx.compose.runtime.*
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.unit.dp
import androidx.compose.ui.window.*
import java.awt.FileDialog
import java.awt.Frame
import java.io.File
import java.util.*


val customRedColor = Color(0xFFB70D1B)
val currentPage = mutableStateOf("accueil")

const val glossaryPath : String = "src/main/resources/projects/"

// Classe pour stocker l'état global
object AppState {
    var selectedProject: Project? = null
}

@OptIn(ExperimentalMaterialApi::class)
@Composable
@Preview
fun app() {
    val languageManager = remember { LanguageManager() }
    MaterialTheme {

        var glossaryDetail: List<Word> by remember { mutableStateOf(emptyList()) }
        var isJavaScriptFileSelected by remember { mutableStateOf(false) }

        var glossaries by remember { mutableStateOf(emptyList<Glossary>()) }

        var selectedGlossary by remember { mutableStateOf<Glossary?>(null) }

        val appState = AppState

        val isEmptySnackbarVisibleState = remember { mutableStateOf(false) }
        val containsSpaceSnackbarVisibleState = remember { mutableStateOf(false) }
        val glossaryAlreadyExistsSnackbarVisibleState = remember { mutableStateOf(false) }






        when (currentPage.value) {
            "accueil" -> {
                homePage(
                    languageManager,
                    onProjectClick = { currentPage.value = "projects" },
                    onCodeToVerifyClick = { currentPage.value = "choixLangage" },
                )
            }

            "glossaires" -> {
                Column(
                    modifier = Modifier.fillMaxSize(),
                    verticalArrangement = Arrangement.Center,
                    horizontalAlignment = Alignment.CenterHorizontally
                ) {
                    Text("Sélectionnez un glossaire", style = MaterialTheme.typography.h5)

                    Spacer(modifier = Modifier.height(16.dp))

                    Box(
                        modifier = Modifier
                            .height(300.dp)
                            .width(250.dp)
                            .padding(10.dp)
                            .border(1.dp, Color.Black)
                    ) {
                        val scrollState = rememberLazyListState()
                        glossaries = loadGlossaries(appState.selectedProject!!)
                        if (glossaries.isEmpty()) {
                            Text("Aucun glossaire n'a été créé pour ce projet")
                        } else {
                            LazyColumn(
                                state = scrollState,
                                modifier = Modifier.padding(10.dp),
                                verticalArrangement = Arrangement.spacedBy(10.dp)
                            ) {
                                println(glossaries)
                                items(glossaries) { glossary ->
                                    Row(
                                        modifier = Modifier.width(200.dp).fillMaxWidth(),
                                        horizontalArrangement = Arrangement.SpaceBetween
                                    ) {
                                        Button(
                                            onClick = {
                                                selectedGlossary = glossary
                                                currentPage.value = "glossaireOptions"
                                            },
                                            modifier = Modifier
                                                .width(150.dp),
                                            colors = ButtonDefaults.buttonColors(
                                                backgroundColor = customRedColor,
                                                contentColor = Color.White
                                            )
                                        ) {
                                            Text(glossary.name)
                                        }

                                        IconButton(
                                            onClick = {
                                                // Handle delete glossary action
                                                glossaries = glossaries.filterNot { it == glossary }
                                                val file =
                                                    File(glossaryPath + (appState.selectedProject?.name) + "/" + glossary.jsonFilePath)
                                                file.delete()
                                            }
                                        ) {
                                            Icon(
                                                imageVector = Icons.Default.Delete,
                                                contentDescription = "Delete Glossary"
                                            )
                                        }
                                    }
                                }
                            }
                        }
                        VerticalScrollbar(
                            modifier = Modifier.align(Alignment.CenterEnd).fillMaxHeight(),
                            adapter = rememberScrollbarAdapter(scrollState)
                        )
                    }

                    Spacer(modifier = Modifier.height(16.dp))
                    // mettre un texte "ou"
                    Text("ou", style = MaterialTheme.typography.h5)
                    Spacer(modifier = Modifier.height(16.dp))


                    Button(
                        onClick = {
                            currentPage.value = "nouveauGlossaire"
                        },
                        modifier = Modifier
                            .width(300.dp),
                        colors = ButtonDefaults.buttonColors(
                            backgroundColor = customRedColor,
                            contentColor = Color.White
                        )
                    ) {
                        Text("Créer un nouveau glossaire")
                    }

                    Spacer(modifier = Modifier.height(6.dp))

                    Button(
                        onClick = {
                            currentPage.value = "accueil"
                        },
                        modifier = Modifier
                            .width(250.dp),
                        colors = ButtonDefaults.buttonColors(
                            backgroundColor = customRedColor,
                            contentColor = Color.White
                        )
                    ) {
                        Text("Retour")
                    }
                }
            }


            // Nouvelle page pour créer un nouveau glossaire
            "nouveauGlossaire" -> {
                var nouveauGlossaireName by remember { mutableStateOf("") }

                Column(
                    modifier = Modifier.fillMaxSize(),
                    verticalArrangement = Arrangement.Center,
                    horizontalAlignment = Alignment.CenterHorizontally
                ) {
                    TextField(
                        value = nouveauGlossaireName,
                        onValueChange = { nouveauGlossaireName = it },
                        label = { Text("Nom du nouveau glossaire") },
                        modifier = Modifier
                            .padding(16.dp)
                            .fillMaxWidth(),
                        colors = TextFieldDefaults.textFieldColors(
                            backgroundColor = Color.White,
                            focusedIndicatorColor = customRedColor,
                            unfocusedIndicatorColor = Color.Gray
                        )
                    )

                    Button(
                        onClick = {
                            // Ajouter le code pour créer un nouveau glossaire avec le nom saisi
                            //first check if the name is not empty, if there is space or if the name already exists
                            if (nouveauGlossaireName.isEmpty()) {
                                println("Veuillez saisir un nom pour le nouveau glossaire")
                                isEmptySnackbarVisibleState.value = true
                                return@Button
                            }
                            if (nouveauGlossaireName.contains(" ")) {
                                println("Le nom du glossaire ne doit pas contenir d'espace")
                                containsSpaceSnackbarVisibleState.value = true
                                return@Button
                            }
                            glossaries.forEach { glossary ->
                                nouveauGlossaireName = nouveauGlossaireName.lowercase(Locale.getDefault())
                                if (glossary.name == nouveauGlossaireName) {
                                    println("Le nom du glossaire existe déjà")
                                    glossaryAlreadyExistsSnackbarVisibleState.value = true
                                    return@Button
                                }
                            }
                            val newGlossary = Glossary(nouveauGlossaireName, "$nouveauGlossaireName.json")
                            glossaries = glossaries + newGlossary
                            //create new json file
                            val newFile = File(glossaryPath + (appState.selectedProject?.name  ) + "/" + newGlossary.jsonFilePath)
                            newFile.createNewFile()
                            //update glossaries list
                            glossaries = loadGlossaries(appState.selectedProject!!)
                            currentPage.value = "glossaires" // Revenir à la liste des glossaires
                        },
                        modifier = Modifier
                            .width(200.dp),
                        colors = ButtonDefaults.buttonColors(
                            backgroundColor = customRedColor,
                            contentColor = Color.White
                        )
                    ) {
                        Text("Créer")
                    }

                    Button(
                        onClick = {
                            currentPage.value = "glossaires"
                        },
                        modifier = Modifier
                            .width(200.dp),
                        colors = ButtonDefaults.buttonColors(
                            backgroundColor = customRedColor,
                            contentColor = Color.White
                        )
                    ) {
                        Text("Annuler")
                    }
                }
            }

            "projects" -> {
                projectsPage(
                    currentPage = currentPage
                )
            }

            "nouveauProjet" ->{
                newProject(
                    currentPage = currentPage
                )
            }

            "glossaireOptions" -> {
                glossaryPage(
                    languageManager,
                    onAddWordClick = { currentPage.value = "formulaire" },
                    onImportClick = {
                        selectFile(setOf("csv", "xlsx")) { filePath ->
                            println("Importing file: $filePath")
                            selectedGlossary?.let { importFile(it, filePath) }
                        }
                    },
                    onExportClick = {
                        val fileDialog = FileDialog(Frame(), "Save as CSV", FileDialog.SAVE)
                        fileDialog.file = selectedGlossary?.name // Initial file name
                        fileDialog.isVisible = true
                        val selectedFile = fileDialog.file
                        val selectedDirectory = fileDialog.directory
                        if (selectedFile != null) {
                            val csvFilePath = "$selectedDirectory$selectedFile"
                            println("Exporting to: $csvFilePath")
                            selectedGlossary?.let { exportToCSV(it, csvFilePath) }
                        } else {
                            println("Export command cancelled by user.")
                        }
                    },
                    onSeeGlossaryClick = {
                        glossaryDetail = selectedGlossary?.let { loadDatasFromFile(it.jsonFilePath) }!!
                        currentPage.value = "glossaireDetail"
                    },
                    glossary = selectedGlossary!!,
                ) { currentPage.value = "glossaires" }
            }
            "glossaireDetail" -> {
                glossaryDetailedPage(
                    glossary = glossaryDetail,
                    onBackClick = { currentPage.value = "glossaireOptions" }
                )
            }

            "formulaire" -> {
                selectedGlossary?.let { formPage(it, onCancelClick = { currentPage.value = "glossaireOptions" }) }
            }

            "choixLangage" -> {
                val pythonExtensions = setOf("py")
                val javaExtensions = setOf("java")
                val jsExtensions = setOf("js", "html")
                choixLangagePage(
                    languageManager,
                    onBackClick = { currentPage.value = "accueil" },
                    onPythonClick = {
                        selectFile(pythonExtensions) { filePath ->
                            println("Python file selected: $filePath") // Change by parser functions
                        }
                    },
                    onJavaClick = {
                        selectFile(javaExtensions) { filePath ->
                            println("Java file selected: $filePath") // Change by parser functions
                        }
                    },
                    onJavaScriptClick = {
                        selectFile(jsExtensions) { filePath ->
                            mostUsedWordList = parser(filePath) // Change by parser functions
                            isJavaScriptFileSelected = true
                        }
                    }
                )
            }
            "occurrence" -> {
                parsedWordsTable(mostUsedWordList, onBackClick = { currentPage.value = "choixLangage" })
            }
        }
        if (isJavaScriptFileSelected) {
            AlertDialog(
                onDismissRequest = {
                    // Handle dialog dismissal if needed
                    isJavaScriptFileSelected = false
                },
                title = {
                    Text("Le fichier a été importé avec succès")
                },
                text = {
                    Text("Vous pouvez maintenant comparer votre code avec le glossaire")
                },
                confirmButton = {
                    Button(
                        onClick = {
                            // Handle the confirm button action
                            isJavaScriptFileSelected = false
                            currentPage.value = "occurrence"
                        },
                        colors = ButtonDefaults.buttonColors(
                            backgroundColor = customRedColor,
                            contentColor = Color.White
                        )
                    ) {
                        Text("Ok")
                    }
                }
            )
        }
        if (isEmptySnackbarVisibleState.value) {
            Snackbar(
                modifier = Modifier.padding(16.dp),
                action = {
                    Button(
                        onClick = { isEmptySnackbarVisibleState.value = false },
                        colors = ButtonDefaults.buttonColors(
                            backgroundColor = customRedColor,
                            contentColor = Color.White
                        )
                    ) {
                        Text("OK")
                    }
                }
            ) {
                Text("Veuillez entrer un nom de glossaire.")
            }
        }
        if (containsSpaceSnackbarVisibleState.value) {
            Snackbar(
                modifier = Modifier.padding(16.dp),
                action = {
                    Button(
                        onClick = { containsSpaceSnackbarVisibleState.value = false },
                        colors = ButtonDefaults.buttonColors(
                            backgroundColor = customRedColor,
                            contentColor = Color.White
                        )
                    ) {
                        Text("OK")
                    }
                }
            ) {
                Text("Veuillez ne pas mettre d'espace dans le nom du glossaire.")
            }
        }
        if (glossaryAlreadyExistsSnackbarVisibleState.value) {
            Snackbar(
                modifier = Modifier.padding(16.dp),
                action = {
                    Button(
                        onClick = { glossaryAlreadyExistsSnackbarVisibleState.value = false },
                        colors = ButtonDefaults.buttonColors(
                            backgroundColor = customRedColor,
                            contentColor = Color.White
                        )
                    ) {
                        Text("OK")
                    }
                }
            ) {
                Text("Ce glossaire existe déjà.")
            }
        }

    }
}

fun loadGlossaries(project: Project): List<Glossary> {
    val glossaries = mutableListOf<Glossary>()
    //Récupérer tous les fichiers json à la racine du projet
    val glossaryFiles = File(glossaryPath + project.name).listFiles { file ->
        file.extension == "json"
    }
    glossaryFiles?.forEach { file ->
        val glossary = Glossary(file.nameWithoutExtension, file.name)
        glossaries.add(glossary)
    }
    println(glossaries)
    return glossaries
}



fun main() = application {
    val state = rememberWindowState(
        placement = WindowPlacement.Floating,
        position = WindowPosition(Alignment.Center),
        isMinimized = false,
        width = 1280.dp,
        height = 720.dp
    )

    Window(
        title = "Quali'Nomme",
        resizable = true,
        state = state,
        icon = painterResource("assets/logo/logo.png"),
        onCloseRequest = ::exitApplication
    ) {
        app()
    }
}
