package main

import androidx.compose.foundation.border
import androidx.compose.foundation.layout.*
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.material.*
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.Delete
import androidx.compose.runtime.*
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.unit.dp
import java.io.File

data class Project(val name: String)

fun loadProjects(): List<Project> {
    val projectsDirectory = File("src/main/resources/projects/")

    return projectsDirectory.listFiles()?.map { Project(it.name) } ?: emptyList()
}

@Composable
fun projectsPage(
    currentPage: MutableState<String>
) {
    val appState = AppState
    var projects: List<Project> by remember { mutableStateOf(loadProjects()) }

    Column(
        modifier = Modifier.fillMaxSize(),
        verticalArrangement = Arrangement.Center,
        horizontalAlignment = Alignment.CenterHorizontally
    ) {
        Text("Sélectionnez un projet", style = MaterialTheme.typography.h5)

        Spacer(modifier = Modifier.height(16.dp))

        LazyColumn(
            modifier = Modifier
                .padding(10.dp)
                .width(200.dp)
                .height(300.dp)
                .border(1.dp, Color.Black),
            verticalArrangement = Arrangement.spacedBy(10.dp)
        ) {
            if (projects.isEmpty()) {
                item {
                    Text("Aucun projet n'a été créé")
                }
            } else {
                items(projects, key = { project -> project.name }) { project ->
                    Row(
                        modifier = Modifier.width(200.dp).fillMaxWidth(),
                        horizontalArrangement = Arrangement.SpaceBetween
                    ) {
                        Button(
                            onClick = {
                                appState.selectedProject = project
                                currentPage.value = "glossaires"

                            },
                            modifier = Modifier
                                .width(150.dp),
                            colors = ButtonDefaults.buttonColors(
                                backgroundColor = customRedColor,
                                contentColor = Color.White
                            )
                        ) {
                            Text(project.name)
                        }

                        IconButton(
                            onClick = {
                                // Handle delete project action
                                projects = projects.filterNot { it.name == project.name }
                                println(projects)
                                val directory = File("src/main/resources/projects/${project.name}/")
                                directory.deleteRecursively()
                                currentPage.value = "projects"
                            }
                        ) {
                            Icon(imageVector = Icons.Default.Delete, contentDescription = "Delete Project")
                        }
                    }
                }
            }
        }

        Spacer(modifier = Modifier.height(16.dp))

        Button(
            onClick = {
                currentPage.value = "nouveauProjet"
            },
            modifier = Modifier
                .width(300.dp),
            colors = ButtonDefaults.buttonColors(
                backgroundColor = customRedColor,
                contentColor = Color.White
            )
        ) {
            Text("Créer un nouveau projet")
        }

        Spacer(modifier = Modifier.height(6.dp))

        Button(
            onClick = {
                currentPage.value = "accueil"
            },
            modifier = Modifier
                .width(250.dp),
            colors = ButtonDefaults.buttonColors(
                backgroundColor = customRedColor,
                contentColor = Color.White
            )
        ) {
            Text("Retour")
        }
    }
}

@Composable
fun newProject(
    currentPage: MutableState<String>
){
    var projectName by remember { mutableStateOf("") }

    Column(
        modifier = Modifier.fillMaxSize(),
        verticalArrangement = Arrangement.Center,
        horizontalAlignment = Alignment.CenterHorizontally
    ){
        Text("Nom du projet", style = MaterialTheme.typography.h5)

        Spacer(modifier = Modifier.height(16.dp))

        TextField(
            value = projectName,
            onValueChange = { projectName = it },
            label = { Text("Nom du projet") },
            modifier = Modifier
                .width(300.dp)
                .fillMaxWidth()
        )

        Spacer(modifier = Modifier.height(16.dp))

        Button(
            onClick = {
                val directory = File("src/main/resources/projects/$projectName/")
                directory.mkdirs()
                println("Project $projectName created")
                currentPage.value = "projects"
            },
            modifier = Modifier
                .width(300.dp),
            colors = ButtonDefaults.buttonColors(
                backgroundColor = customRedColor,
                contentColor = Color.White
            )
        ) {
            Text("Créer")
        }

        Spacer(modifier = Modifier.height(6.dp))

        Button(
            onClick = {
                currentPage.value = "projects"
            },
            modifier = Modifier
                .width(250.dp),
            colors = ButtonDefaults.buttonColors(
                backgroundColor = customRedColor,
                contentColor = Color.White
            )
        ) {
            Text("Retour")
        }
    }
}




